@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -42,7 +42,6 @@ def custom_headers(url):
     if dnt_config is not None:
         dnt = b'1' if dnt_config else b'0'
         headers[b'DNT'] = dnt
-        headers[b'X-Do-Not-Track'] = dnt
 
     conf_headers = config.instance.get('content.headers.custom', url=url)
     for header, value in conf_headers.items():
@@ -131,19 +130,21 @@ def javascript_alert(url, js_msg, abort_on, *, escape_msg=True):
                 abort_on=abort_on, url=urlstr)
 
 
+# Needs to line up with the values allowed for the
+# content.javascript.log setting.
+_JS_LOGMAP = {
+    'none': lambda arg: None,
+    'debug': log.js.debug,
+    'info': log.js.info,
+    'warning': log.js.warning,
+    'error': log.js.error,
+}
+
+
 def javascript_log_message(level, source, line, msg):
     """Display a JavaScript log message."""
     logstring = "[{}:{}] {}".format(source, line, msg)
-    # Needs to line up with the values allowed for the
-    # content.javascript.log setting.
-    logmap = {
-        'none': lambda arg: None,
-        'debug': log.js.debug,
-        'info': log.js.info,
-        'warning': log.js.warning,
-        'error': log.js.error,
-    }
-    logger = logmap[config.val.content.javascript.log[level.name]]
+    logger = _JS_LOGMAP[config.cache['content.javascript.log'][level.name]]
     logger(logstring)
 
 
@@ -222,11 +223,12 @@ def feature_permission(url, option, msg, yes_action, no_action, abort_on,
                 html.escape(url.toDisplayString()), msg)
         else:
             urlstr = None
+            option = None  # For message.ask/confirm_async
             text = "Allow the website to {}?".format(msg)
 
         if blocking:
             answer = message.ask(abort_on=abort_on, title='Permission request',
-                                 text=text, url=urlstr,
+                                 text=text, url=urlstr, option=option,
                                  mode=usertypes.PromptMode.yesno)
             if answer:
                 yes_action()
@@ -237,7 +239,8 @@ def feature_permission(url, option, msg, yes_action, no_action, abort_on,
             return message.confirm_async(
                 yes_action=yes_action, no_action=no_action,
                 cancel_action=no_action, abort_on=abort_on,
-                title='Permission request', text=text, url=urlstr)
+                title='Permission request', text=text, url=urlstr,
+                option=option)
     elif config_val:
         yes_action()
         return None
@@ -262,7 +265,7 @@ def get_tab(win_id, target):
     elif target == usertypes.ClickTarget.window:
         tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                     window=win_id)
-        window = mainwindow.MainWindow(private=tabbed_browser.private)
+        window = mainwindow.MainWindow(private=tabbed_browser.is_private)
         window.show()
         win_id = window.win_id
         bg_tab = False
@@ -274,7 +277,7 @@ def get_tab(win_id, target):
     return tabbed_browser.tabopen(url=None, background=bg_tab)
 
 
-def get_user_stylesheet():
+def get_user_stylesheet(searching=False):
     """Get the combined user-stylesheet."""
     css = ''
     stylesheets = config.val.content.user_stylesheets
@@ -283,7 +286,8 @@ def get_user_stylesheet():
         with open(filename, 'r', encoding='utf-8') as f:
             css += f.read()
 
-    if not config.val.scrolling.bar:
+    if (config.val.scrolling.bar == 'never' or
+            config.val.scrolling.bar == 'when-searching' and not searching):
         css += '\nhtml > ::-webkit-scrollbar { width: 0px; height: 0px; }'
 
     return css
@@ -305,12 +309,23 @@ def netrc_authentication(url, authenticator):
         # os.environ. We don't want to log that, so we prevent it
         # altogether.
         return False
-    user, password = None, None
+
+    user = None
+    password = None
+    authenticators = None
+
     try:
         net = netrc.netrc(config.val.content.netrc_file)
-        authenticators = net.authenticators(url.host())
-        if authenticators is not None:
-            (user, _account, password) = authenticators
+
+        if url.port() != -1:
+            authenticators = net.authenticators(
+                "{}:{}".format(url.host(), url.port()))
+
+        if not authenticators:
+            authenticators = net.authenticators(url.host())
+
+        if authenticators:
+            user, _account, password = authenticators
     except FileNotFoundError:
         log.misc.debug("No .netrc file found")
     except OSError as e:
