@@ -1,5 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
+# Copyright 2017-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 # Copyright 2017-2018 Michal Siedlaczek <michal.siedlaczek@gmail.com>
 
 # This file is part of qutebrowser.
@@ -28,12 +29,21 @@ import shutil
 from PyQt5.QtCore import QLibraryInfo
 from qutebrowser.utils import log, message, standarddir, qtutils
 
-dict_version_re = re.compile(r".+-(?P<version>[0-9]+-[0-9]+?)\.bdic")
+_DICT_VERSION_RE = re.compile(r".+-(?P<version>[0-9]+-[0-9]+?)\.bdic")
+
+
+def can_use_data_path():
+    """Whether the current Qt version can use a customized path.
+
+    Qt >= 5.10 understands QTWEBENGINE_DICTIONARIES_PATH which means we don't
+    need to put them to a fixed root-only folder.
+    """
+    return qtutils.version_check('5.10', compiled=False)
 
 
 def version(filename):
     """Extract the version number from the dictionary file name."""
-    match = dict_version_re.match(filename)
+    match = _DICT_VERSION_RE.fullmatch(filename)
     if match is None:
         message.warning(
             "Found a dictionary with a malformed name: {}".format(filename))
@@ -43,7 +53,7 @@ def version(filename):
 
 def dictionary_dir(old=False):
     """Return the path (str) to the QtWebEngine's dictionaries directory."""
-    if qtutils.version_check('5.10', compiled=False) and not old:
+    if can_use_data_path() and not old:
         datapath = standarddir.data()
     else:
         datapath = QLibraryInfo.location(QLibraryInfo.DataPath)
@@ -77,12 +87,12 @@ def local_filename(code):
     number or None if the dictionary is not installed.
     """
     all_installed = local_files(code)
-    return os.path.splitext(all_installed[0])[0] if all_installed else None
+    return all_installed[0] if all_installed else None
 
 
 def init():
     """Initialize the dictionary path if supported."""
-    if qtutils.version_check('5.10', compiled=False):
+    if can_use_data_path():
         new_dir = dictionary_dir()
         old_dir = dictionary_dir(old=True)
         os.environ['QTWEBENGINE_DICTIONARIES_PATH'] = new_dir
