@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -23,9 +23,8 @@ import re
 import functools
 import xml.etree.ElementTree
 
-from PyQt5.QtCore import (pyqtSlot, Qt, QEvent, QUrl, QPoint, QTimer, QSizeF,
-                          QSize)
-from PyQt5.QtGui import QKeyEvent, QIcon
+from PyQt5.QtCore import pyqtSlot, Qt, QUrl, QPoint, QTimer, QSizeF, QSize
+from PyQt5.QtGui import QIcon
 from PyQt5.QtWebKitWidgets import QWebPage, QWebFrame
 from PyQt5.QtWebKit import QWebSettings
 from PyQt5.QtPrintSupport import QPrinter
@@ -84,8 +83,8 @@ class WebKitSearch(browsertab.AbstractSearch):
 
     """QtWebKit implementations related to searching on the page."""
 
-    def __init__(self, parent=None):
-        super().__init__(parent)
+    def __init__(self, tab, parent=None):
+        super().__init__(tab, parent)
         self._flags = QWebPage.FindFlags(0)
 
     def _call_cb(self, callback, found, text, flags, caller):
@@ -115,14 +114,18 @@ class WebKitSearch(browsertab.AbstractSearch):
         if callback is not None:
             QTimer.singleShot(0, functools.partial(callback, found))
 
+        self.finished.emit(found)
+
     def clear(self):
+        if self.search_displayed:
+            self.cleared.emit()
         self.search_displayed = False
         # We first clear the marked text, then the highlights
         self._widget.findText('')
         self._widget.findText('', QWebPage.HighlightAllOccurrences)
 
-    def search(self, text, *, ignore_case='never', reverse=False,
-               result_cb=None):
+    def search(self, text, *, ignore_case=usertypes.IgnoreCase.never,
+               reverse=False, result_cb=None):
         # Don't go to next entry on duplicate search
         if self.text == text and self.search_displayed:
             log.webview.debug("Ignoring duplicate search request"
@@ -348,7 +351,16 @@ class WebKitCaret(browsertab.AbstractCaret):
     def selection(self, callback):
         callback(self._widget.selectedText())
 
-    def follow_selected(self, *, tab=False):
+    def reverse_selection(self):
+        self._tab.run_js_async("""{
+            const sel = window.getSelection();
+            sel.setBaseAndExtent(
+                sel.extentNode, sel.extentOffset, sel.baseNode,
+                sel.baseOffset
+            );
+        }""")
+
+    def _follow_selected(self, *, tab=False):
         if QWebSettings.globalSettings().testAttribute(
                 QWebSettings.JavascriptEnabled):
             if tab:
@@ -387,7 +399,13 @@ class WebKitCaret(browsertab.AbstractCaret):
                 if tab:
                     self._tab.new_tab_requested.emit(url)
                 else:
-                    self._tab.openurl(url)
+                    self._tab.load_url(url)
+
+    def follow_selected(self, *, tab=False):
+        try:
+            self._follow_selected(tab=tab)
+        finally:
+            self.follow_selected_done.emit()
 
 
 class WebKitZoom(browsertab.AbstractZoom):
@@ -464,7 +482,7 @@ class WebKitScroller(browsertab.AbstractScroller):
             if (getter is not None and
                     frame.scrollBarValue(direction) == getter(direction)):
                 return
-            self._tab.key_press(key)
+            self._tab.fake_key_press(key)
 
     def up(self, count=1):
         self._key_press(Qt.Key_Up, count, 'scrollBarMinimum', Qt.Vertical)
@@ -499,35 +517,19 @@ class WebKitScroller(browsertab.AbstractScroller):
         return self.pos_px().y() >= frame.scrollBarMaximum(Qt.Vertical)
 
 
-class WebKitHistory(browsertab.AbstractHistory):
+class WebKitHistoryPrivate(browsertab.AbstractHistoryPrivate):
 
-    """QtWebKit implementations related to page history."""
-
-    def current_idx(self):
-        return self._history.currentItemIndex()
-
-    def can_go_back(self):
-        return self._history.canGoBack()
-
-    def can_go_forward(self):
-        return self._history.canGoForward()
-
-    def _item_at(self, i):
-        return self._history.itemAt(i)
-
-    def _go_to_item(self, item):
-        self._tab.predicted_navigation.emit(item.url())
-        self._history.goToItem(item)
+    """History-related methods which are not part of the extension API."""
 
     def serialize(self):
         return qtutils.serialize(self._history)
 
     def deserialize(self, data):
-        return qtutils.deserialize(data, self._history)
+        qtutils.deserialize(data, self._history)
 
     def load_items(self, items):
         if items:
-            self._tab.predicted_navigation.emit(items[-1].url)
+            self._tab.before_load_started.emit(items[-1].url)
 
         stream, _data, user_data = tabhistory.serialize(items)
         qtutils.deserialize_stream(stream, self._history)
@@ -543,11 +545,43 @@ class WebKitHistory(browsertab.AbstractHistory):
                     self._tab.scroller.to_point, cur_data['scroll-pos']))
 
 
+class WebKitHistory(browsertab.AbstractHistory):
+
+    """QtWebKit implementations related to page history."""
+
+    def __init__(self, tab):
+        super().__init__(tab)
+        self.private_api = WebKitHistoryPrivate(tab)
+
+    def __len__(self):
+        return len(self._history)
+
+    def __iter__(self):
+        return iter(self._history.items())
+
+    def current_idx(self):
+        return self._history.currentItemIndex()
+
+    def can_go_back(self):
+        return self._history.canGoBack()
+
+    def can_go_forward(self):
+        return self._history.canGoForward()
+
+    def _item_at(self, i):
+        return self._history.itemAt(i)
+
+    def _go_to_item(self, item):
+        self._tab.before_load_started.emit(item.url())
+        self._history.goToItem(item)
+
+
 class WebKitElements(browsertab.AbstractElements):
 
     """QtWebKit implemementations related to elements on the page."""
 
-    def find_css(self, selector, callback, *, only_visible=False):
+    def find_css(self, selector, callback, error_cb, *, only_visible=False):
+        utils.unused(error_cb)
         mainframe = self._widget.page().mainFrame()
         if mainframe is None:
             raise browsertab.WebTabError("No frame focused!")
@@ -576,7 +610,7 @@ class WebKitElements(browsertab.AbstractElements):
         # Escape non-alphanumeric characters in the selector
         # https://www.w3.org/TR/CSS2/syndata.html#value-def-identifier
         elem_id = re.sub(r'[^a-zA-Z0-9_-]', r'\\\g<0>', elem_id)
-        self.find_css('#' + elem_id, find_id_cb)
+        self.find_css('#' + elem_id, find_id_cb, error_cb=lambda exc: None)
 
     def find_focused(self, callback):
         frame = self._widget.page().currentFrame()
@@ -631,7 +665,7 @@ class WebKitAudio(browsertab.AbstractAudio):
 
     """Dummy handling of audio status for QtWebKit."""
 
-    def set_muted(self, muted: bool):
+    def set_muted(self, muted: bool, override: bool = False) -> None:
         raise browsertab.WebTabError('Muting is not supported on QtWebKit!')
 
     def is_muted(self):
@@ -641,27 +675,49 @@ class WebKitAudio(browsertab.AbstractAudio):
         return False
 
 
+class WebKitTabPrivate(browsertab.AbstractTabPrivate):
+
+    """QtWebKit-related methods which aren't part of the public API."""
+
+    def networkaccessmanager(self):
+        return self._widget.page().networkAccessManager()
+
+    def user_agent(self):
+        page = self._widget.page()
+        return page.userAgentForUrl(self._tab.url())
+
+    def clear_ssl_errors(self):
+        self.networkaccessmanager().clear_all_ssl_errors()
+
+    def event_target(self):
+        return self._widget
+
+    def shutdown(self):
+        self._widget.shutdown()
+
+
 class WebKitTab(browsertab.AbstractTab):
 
     """A QtWebKit tab in the browser."""
 
     def __init__(self, *, win_id, mode_manager, private, parent=None):
-        super().__init__(win_id=win_id, mode_manager=mode_manager,
-                         private=private, parent=parent)
+        super().__init__(win_id=win_id, private=private, parent=parent)
         widget = webview.WebView(win_id=win_id, tab_id=self.tab_id,
                                  private=private, tab=self)
         if private:
             self._make_private(widget)
-        self.history = WebKitHistory(self)
-        self.scroller = WebKitScroller(self, parent=self)
+        self.history = WebKitHistory(tab=self)
+        self.scroller = WebKitScroller(tab=self, parent=self)
         self.caret = WebKitCaret(mode_manager=mode_manager,
                                  tab=self, parent=self)
         self.zoom = WebKitZoom(tab=self, parent=self)
-        self.search = WebKitSearch(parent=self)
+        self.search = WebKitSearch(tab=self, parent=self)
         self.printing = WebKitPrinting(tab=self)
         self.elements = WebKitElements(tab=self)
         self.action = WebKitAction(tab=self)
-        self.audio = WebKitAudio(parent=self)
+        self.audio = WebKitAudio(tab=self, parent=self)
+        self.private_api = WebKitTabPrivate(mode_manager=mode_manager,
+                                            tab=self)
         # We're assigning settings in _set_widget
         self.settings = webkitsettings.WebKitSettings(settings=None)
         self._set_widget(widget)
@@ -669,17 +725,18 @@ class WebKitTab(browsertab.AbstractTab):
         self.backend = usertypes.Backend.QtWebKit
 
     def _install_event_filter(self):
-        self._widget.installEventFilter(self._mouse_event_filter)
+        self._widget.installEventFilter(self._tab_event_filter)
 
     def _make_private(self, widget):
         settings = widget.settings()
         settings.setAttribute(QWebSettings.PrivateBrowsingEnabled, True)
 
-    def openurl(self, url, *, predict=True):
-        self._openurl_prepare(url, predict=predict)
-        self._widget.openurl(url)
+    def load_url(self, url, *, emit_before_load_started=True):
+        self._load_url_prepare(
+            url, emit_before_load_started=emit_before_load_started)
+        self._widget.load(url)
 
-    def url(self, requested=False):
+    def url(self, *, requested=False):
         frame = self._widget.page().mainFrame()
         if requested:
             return frame.requestedUrl()
@@ -704,9 +761,6 @@ class WebKitTab(browsertab.AbstractTab):
     def icon(self):
         return self._widget.icon()
 
-    def shutdown(self):
-        self._widget.shutdown()
-
     def reload(self, *, force=False):
         if force:
             action = QWebPage.ReloadAndBypassCache
@@ -720,39 +774,28 @@ class WebKitTab(browsertab.AbstractTab):
     def title(self):
         return self._widget.title()
 
-    def clear_ssl_errors(self):
-        self.networkaccessmanager().clear_all_ssl_errors()
-
-    def key_press(self, key, modifier=Qt.NoModifier):
-        press_evt = QKeyEvent(QEvent.KeyPress, key, modifier, 0, 0, 0)
-        release_evt = QKeyEvent(QEvent.KeyRelease, key, modifier,
-                                0, 0, 0)
-        self.send_event(press_evt)
-        self.send_event(release_evt)
-
     @pyqtSlot()
     def _on_history_trigger(self):
         url = self.url()
         requested_url = self.url(requested=True)
-        self.add_history_item.emit(url, requested_url, self.title())
+        self.history_item_triggered.emit(url, requested_url, self.title())
 
     def set_html(self, html, base_url=QUrl()):
         self._widget.setHtml(html, base_url)
 
-    def networkaccessmanager(self):
-        return self._widget.page().networkAccessManager()
-
-    def user_agent(self):
-        page = self._widget.page()
-        return page.userAgentForUrl(self.url())
-
     @pyqtSlot()
     def _on_load_started(self):
         super()._on_load_started()
-        self.networkaccessmanager().netrc_used = False
+        nam = self._widget.page().networkAccessManager()
+        nam.netrc_used = False
         # Make sure the icon is cleared when navigating to a page without one.
         self.icon_changed.emit(QIcon())
 
+    @pyqtSlot(bool)
+    def _on_load_finished(self, ok: bool) -> None:
+        super()._on_load_finished(ok)
+        self._update_load_status(ok)
+
     @pyqtSlot()
     def _on_frame_load_finished(self):
         """Make sure we emit an appropriate status when loading finished.
@@ -801,7 +844,7 @@ class WebKitTab(browsertab.AbstractTab):
         if (navigation.navigation_type == navigation.Type.link_clicked and
                 target != usertypes.ClickTarget.normal):
             tab = shared.get_tab(self.win_id, target)
-            tab.openurl(navigation.url)
+            tab.load_url(navigation.url)
             self.data.open_target = usertypes.ClickTarget.normal
             navigation.accepted = False
 
@@ -831,6 +874,3 @@ class WebKitTab(browsertab.AbstractTab):
         frame.contentsSizeChanged.connect(self._on_contents_size_changed)
         frame.initialLayoutCompleted.connect(self._on_history_trigger)
         page.navigation_request.connect(self._on_navigation_request)
-
-    def event_target(self):
-        return self._widget
