@@ -17,20 +17,26 @@
 # You should have received a copy of the GNU General Public License
 # along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
 
-"""A completion category that queries the SQL History store."""
+"""A completion category that queries the SQL history store."""
+
+import typing  # pylint: disable=unused-import,useless-suppression
 
 from PyQt5.QtSql import QSqlQueryModel
+from PyQt5.QtWidgets import QWidget
 
 from qutebrowser.misc import sql
-from qutebrowser.utils import debug, message
+from qutebrowser.utils import debug, message, log
 from qutebrowser.config import config
+from qutebrowser.completion.models import util
 
 
 class HistoryCategory(QSqlQueryModel):
 
-    """A completion category that queries the SQL History store."""
+    """A completion category that queries the SQL history store."""
 
-    def __init__(self, *, delete_func=None, parent=None):
+    def __init__(self, *,
+                 delete_func: util.DeleteFuncType = None,
+                 parent: QWidget = None) -> None:
         """Create a new History completion category."""
         super().__init__(parent=parent)
         self.name = "History"
@@ -39,6 +45,7 @@ class HistoryCategory(QSqlQueryModel):
         # advertise that this model filters by URL and title
         self.columns_to_filter = [0, 1]
         self.delete_func = delete_func
+        self._empty_prefix = None  # type: typing.Optional[str]
 
     def _atime_expr(self):
         """If max_items is set, return an expression to limit the query."""
@@ -67,6 +74,15 @@ class HistoryCategory(QSqlQueryModel):
         Args:
             pattern: string pattern to filter by.
         """
+        raw_pattern = pattern
+        if (self._empty_prefix is not None and raw_pattern.startswith(
+                self._empty_prefix)):
+            log.sql.debug('Skipping query on {} due to '
+                          'prefix {} returning nothing.'
+                          .format(raw_pattern, self._empty_prefix))
+            return
+        self._empty_prefix = None
+
         # escape to treat a user input % or _ as a literal, not a wildcard
         pattern = pattern.replace('%', '\\%')
         pattern = pattern.replace('_', '\\_')
@@ -74,11 +90,11 @@ class HistoryCategory(QSqlQueryModel):
 
         # build a where clause to match all of the words in any order
         # given the search term "a b", the WHERE clause would be:
-        # ((url || ' ' || title) LIKE '%a%') AND
-        # ((url || ' ' || title) LIKE '%b%')
+        # (url LIKE '%a%' OR title LIKE '%a%') AND
+        # (url LIKE '%b%' OR title LIKE '%b%')
         where_clause = ' AND '.join(
-            "(url || ' ' || title) LIKE :{} escape '\\'".format(i)
-            for i in range(len(words)))
+            "(url LIKE :{val} escape '\\' OR title LIKE :{val} escape '\\')"
+            .format(val=i) for i in range(len(words)))
 
         # replace ' in timestamp-format to avoid breaking the query
         timestamp_format = config.val.completion.timestamp_format or ''
@@ -95,7 +111,7 @@ class HistoryCategory(QSqlQueryModel):
                     "SELECT url, title, {}".format(timefmt),
                     "FROM CompletionHistory",
                     # the incoming pattern will have literal % and _ escaped we
-                    # need to tell sql to treat '\' as an escape character
+                    # need to tell SQL to treat '\' as an escape character
                     'WHERE ({})'.format(where_clause),
                     self._atime_expr(),
                     "ORDER BY last_atime DESC",
@@ -104,16 +120,18 @@ class HistoryCategory(QSqlQueryModel):
             with debug.log_time('sql', 'Running completion query'):
                 self._query.run(**{
                     str(i): w for i, w in enumerate(words)})
-        except sql.SqlKnownError as e:
+        except sql.KnownError as e:
             # Sometimes, the query we built up was invalid, for example,
             # due to a large amount of words.
             # Also catches failures in the DB we can't solve.
-            message.error("Error with SQL Query: {}".format(e.text()))
+            message.error("Error with SQL query: {}".format(e.text()))
             return
         self.setQuery(self._query.query)
+        if not self.rowCount() and not self.canFetchMore():
+            self._empty_prefix = raw_pattern
 
     def removeRows(self, row, _count, _parent=None):
-        """Override QAbstractItemModel::removeRows to re-run sql query."""
+        """Override QAbstractItemModel::removeRows to re-run SQL query."""
         # re-run query to reload updated table
         with debug.log_time('sql', 'Re-running completion query post-delete'):
             self._query.run()
