@@ -30,8 +30,8 @@ class ConfigCache:
     """A 'high-performance' cache for the config system.
 
     Useful for areas which call out to the config system very frequently, DO
-    NOT modify the value returned, DO NOT require per-url settings, do not
-    change frequently, and do not require partially 'expanded' config paths.
+    NOT modify the value returned, DO NOT require per-url settings, and do not
+    require partially 'expanded' config paths.
 
     If any of these requirements are broken, you will get incorrect or slow
     behavior.
@@ -43,12 +43,12 @@ class ConfigCache:
 
     def _on_config_changed(self, attr: str) -> None:
         if attr in self._cache:
-            self._cache[attr] = config.instance.get(attr)
+            del self._cache[attr]
 
     def __getitem__(self, attr: str) -> typing.Any:
         try:
             return self._cache[attr]
         except KeyError:
             assert not config.instance.get_opt(attr).supports_pattern
-            self._cache[attr] = config.instance.get(attr)
-            return self._cache[attr]
+            result = self._cache[attr] = config.instance.get(attr)
+            return result
