@@ -20,15 +20,120 @@
 """Base class for vim-like key sequence parser."""
 
 import string
+import types
+import typing
 
-from PyQt5.QtCore import pyqtSignal, QObject
-from PyQt5.QtGui import QKeySequence
+import attr
+from PyQt5.QtCore import pyqtSignal, QObject, Qt
+from PyQt5.QtGui import QKeySequence, QKeyEvent
+from PyQt5.QtWidgets import QWidget
 
 from qutebrowser.config import config
 from qutebrowser.utils import usertypes, log, utils
 from qutebrowser.keyinput import keyutils
 
 
+@attr.s(frozen=True)
+class MatchResult:
+
+    """The result of matching a keybinding."""
+
+    match_type = attr.ib()  # type: QKeySequence.SequenceMatch
+    command = attr.ib()  # type: typing.Optional[str]
+    sequence = attr.ib()  # type: keyutils.KeySequence
+
+    def __attrs_post_init__(self):
+        if self.match_type == QKeySequence.ExactMatch:
+            assert self.command is not None
+        else:
+            assert self.command is None
+
+
+class BindingTrie:
+
+    """Helper class for key parser. Represents a set of bindings.
+
+    Every BindingTree will either contain children or a command (for leaf
+    nodes). The only exception is the root BindingNode, if there are no
+    bindings at all.
+
+    From the outside, this class works similar to a mapping of
+    keyutils.KeySequence to str. Doing trie[sequence] = 'command' adds a
+    binding, and so does calling .update() with a mapping. Additionally, a
+    "matches" method can be used to do partial matching.
+
+    However, some mapping methods are not (yet) implemented:
+    - __getitem__ (use matches() instead)
+    - __len__
+    - __iter__
+    - __delitem__
+
+    Attributes:
+        children: A mapping from KeyInfo to children BindingTries.
+        command: Command associated with this trie node.
+    """
+
+    __slots__ = 'children', 'command'
+
+    def __init__(self) -> None:
+        self.children = {
+        }  # type: typing.MutableMapping[keyutils.KeyInfo, BindingTrie]
+        self.command = None  # type: typing.Optional[str]
+
+    def __setitem__(self, sequence: keyutils.KeySequence,
+                    command: str) -> None:
+        node = self
+        for key in sequence:
+            if key not in node.children:
+                node.children[key] = BindingTrie()
+            node = node.children[key]
+
+        node.command = command
+
+    def __contains__(self, sequence: keyutils.KeySequence) -> bool:
+        return self.matches(sequence).match_type == QKeySequence.ExactMatch
+
+    def __repr__(self) -> str:
+        return utils.get_repr(self, children=self.children,
+                              command=self.command)
+
+    def update(self, mapping: typing.Mapping) -> None:
+        """Add data from the given mapping to the trie."""
+        for key in mapping:
+            self[key] = mapping[key]
+
+    def matches(self, sequence: keyutils.KeySequence) -> MatchResult:
+        """Try to match a given keystring with any bound keychain.
+
+        Args:
+            sequence: The key sequence to match.
+
+        Return:
+            A MatchResult object.
+        """
+        node = self
+        for key in sequence:
+            try:
+                node = node.children[key]
+            except KeyError:
+                return MatchResult(match_type=QKeySequence.NoMatch,
+                                   command=None,
+                                   sequence=sequence)
+
+        if node.command is not None:
+            return MatchResult(match_type=QKeySequence.ExactMatch,
+                               command=node.command,
+                               sequence=sequence)
+        elif node.children:
+            return MatchResult(match_type=QKeySequence.PartialMatch,
+                               command=None,
+                               sequence=sequence)
+        else:  # This can only happen when there are no bindings at all.
+            return MatchResult(match_type=QKeySequence.NoMatch,
+                               command=None,
+                               sequence=sequence)
+
+
 class BaseKeyParser(QObject):
 
     """Parser for vim-like key sequences and shortcuts.
@@ -68,20 +173,21 @@ class BaseKeyParser(QObject):
     do_log = True
     passthrough = False
 
-    def __init__(self, win_id, parent=None, supports_count=True):
+    def __init__(self, win_id: int, parent: QWidget = None,
+                 supports_count: bool = True) -> None:
         super().__init__(parent)
         self._win_id = win_id
         self._modename = None
         self._sequence = keyutils.KeySequence()
         self._count = ''
         self._supports_count = supports_count
-        self.bindings = {}
+        self.bindings = BindingTrie()
         config.instance.changed.connect(self._on_config_changed)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return utils.get_repr(self, supports_count=self._supports_count)
 
-    def _debug_log(self, message):
+    def _debug_log(self, message: str) -> None:
         """Log a message to the debug log if logging is active.
 
         Args:
@@ -90,7 +196,7 @@ class BaseKeyParser(QObject):
         if self.do_log:
             log.keyboard.debug(message)
 
-    def _match_key(self, sequence):
+    def _match_key(self, sequence: keyutils.KeySequence) -> MatchResult:
         """Try to match a given keystring with any bound keychain.
 
         Args:
@@ -104,38 +210,31 @@ class BaseKeyParser(QObject):
         """
         assert sequence
         assert not isinstance(sequence, str)
-        result = QKeySequence.NoMatch
+        return self.bindings.matches(sequence)
 
-        for seq, cmd in self.bindings.items():
-            assert not isinstance(seq, str), seq
-            match = sequence.matches(seq)
-            if match == QKeySequence.ExactMatch:
-                return match, cmd
-            elif match == QKeySequence.PartialMatch:
-                result = QKeySequence.PartialMatch
-
-        return result, None
-
-    def _match_without_modifiers(self, sequence):
+    def _match_without_modifiers(
+            self, sequence: keyutils.KeySequence) -> MatchResult:
         """Try to match a key with optional modifiers stripped."""
         self._debug_log("Trying match without modifiers")
         sequence = sequence.strip_modifiers()
-        match, binding = self._match_key(sequence)
-        return match, binding, sequence
+        return self._match_key(sequence)
 
-    def _match_key_mapping(self, sequence):
+    def _match_key_mapping(
+            self, sequence: keyutils.KeySequence) -> MatchResult:
         """Try to match a key in bindings.key_mappings."""
         self._debug_log("Trying match with key_mappings")
-        mapped = sequence.with_mappings(config.val.bindings.key_mappings)
+        mapped = sequence.with_mappings(
+            types.MappingProxyType(config.cache['bindings.key_mappings']))
         if sequence != mapped:
             self._debug_log("Mapped {} -> {}".format(
                 sequence, mapped))
-            match, binding = self._match_key(mapped)
-            sequence = mapped
-            return match, binding, sequence
-        return QKeySequence.NoMatch, None, sequence
+            return self._match_key(mapped)
+        return MatchResult(match_type=QKeySequence.NoMatch,
+                           command=None,
+                           sequence=sequence)
 
-    def _match_count(self, sequence, dry_run):
+    def _match_count(self, sequence: keyutils.KeySequence,
+                     dry_run: bool) -> bool:
         """Try to match a key as count."""
         txt = str(sequence[-1])  # To account for sequences changed above.
         if (txt in string.digits and self._supports_count and
@@ -148,7 +247,8 @@ class BaseKeyParser(QObject):
             return True
         return False
 
-    def handle(self, e, *, dry_run=False):
+    def handle(self, e: QKeyEvent, *,
+               dry_run: bool = False) -> QKeySequence.SequenceMatch:
         """Handle a new keypress.
 
         Separate the keypress into count/command, then check if it matches
@@ -163,7 +263,7 @@ class BaseKeyParser(QObject):
         Return:
             A QKeySequence match.
         """
-        key = e.key()
+        key = Qt.Key(e.key())
         txt = str(keyutils.KeyInfo.from_event(e))
         self._debug_log("Got key: 0x{:x} / modifiers: 0x{:x} / text: '{}' / "
                         "dry_run {}".format(key, int(e.modifiers()), txt,
@@ -180,45 +280,47 @@ class BaseKeyParser(QObject):
             self.clear_keystring()
             return QKeySequence.NoMatch
 
-        match, binding = self._match_key(sequence)
-        if match == QKeySequence.NoMatch:
-            match, binding, sequence = self._match_without_modifiers(sequence)
-        if match == QKeySequence.NoMatch:
-            match, binding, sequence = self._match_key_mapping(sequence)
-        if match == QKeySequence.NoMatch:
-            was_count = self._match_count(sequence, dry_run)
+        result = self._match_key(sequence)
+        if result.match_type == QKeySequence.NoMatch:
+            result = self._match_without_modifiers(result.sequence)
+        if result.match_type == QKeySequence.NoMatch:
+            result = self._match_key_mapping(result.sequence)
+        if result.match_type == QKeySequence.NoMatch:
+            was_count = self._match_count(result.sequence, dry_run)
             if was_count:
                 return QKeySequence.ExactMatch
 
         if dry_run:
-            return match
+            return result.match_type
 
-        self._sequence = sequence
+        self._sequence = result.sequence
 
-        if match == QKeySequence.ExactMatch:
+        if result.match_type == QKeySequence.ExactMatch:
+            assert result.command is not None
             self._debug_log("Definitive match for '{}'.".format(
-                sequence))
+                result.sequence))
             count = int(self._count) if self._count else None
             self.clear_keystring()
-            self.execute(binding, count)
-        elif match == QKeySequence.PartialMatch:
+            self.execute(result.command, count)
+        elif result.match_type == QKeySequence.PartialMatch:
             self._debug_log("No match for '{}' (added {})".format(
-                sequence, txt))
+                result.sequence, txt))
             self.keystring_updated.emit(self._count + str(sequence))
-        elif match == QKeySequence.NoMatch:
+        elif result.match_type == QKeySequence.NoMatch:
             self._debug_log("Giving up with '{}', no matches".format(
-                sequence))
+                result.sequence))
             self.clear_keystring()
         else:
-            raise utils.Unreachable("Invalid match value {!r}".format(match))
+            raise utils.Unreachable("Invalid match value {!r}".format(
+                result.match_type))
 
-        return match
+        return result.match_type
 
     @config.change_filter('bindings')
-    def _on_config_changed(self):
+    def _on_config_changed(self) -> None:
         self._read_config()
 
-    def _read_config(self, modename=None):
+    def _read_config(self, modename: str = None) -> None:
         """Read the configuration.
 
         Config format: key = command, e.g.:
@@ -234,14 +336,14 @@ class BaseKeyParser(QObject):
             modename = self._modename
         else:
             self._modename = modename
-        self.bindings = {}
+        self.bindings = BindingTrie()
 
         for key, cmd in config.key_instance.get_bindings_for(modename).items():
             assert not isinstance(key, str), key
             assert cmd
             self.bindings[key] = cmd
 
-    def execute(self, cmdstr, count=None):
+    def execute(self, cmdstr: str, count: int = None) -> None:
         """Handle a completed keychain.
 
         Args:
@@ -250,7 +352,7 @@ class BaseKeyParser(QObject):
         """
         raise NotImplementedError
 
-    def clear_keystring(self):
+    def clear_keystring(self) -> None:
         """Clear the currently entered key sequence."""
         if self._sequence:
             self._debug_log("Clearing keystring (was: {}).".format(
