@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -21,13 +21,17 @@
 
 import os
 import os.path
+import typing
+import functools
 import contextlib
 import html
 
 import jinja2
+import jinja2.nodes
 from PyQt5.QtCore import QUrl
 
-from qutebrowser.utils import utils, urlutils, log
+from qutebrowser.utils import utils, urlutils, log, qtutils, javascript
+from qutebrowser.misc import debugcachestats
 
 
 html_fallback = """
@@ -85,6 +89,8 @@ class Environment(jinja2.Environment):
         self.globals['resource_url'] = self._resource_url
         self.globals['file_url'] = urlutils.file_url
         self.globals['data_url'] = self._data_url
+        self.globals['qcolor_to_qsscolor'] = qtutils.qcolor_to_qsscolor
+        self.filters['js_string_escape'] = javascript.string_escape
         self._autoescape = True
 
     @contextlib.contextmanager
@@ -126,3 +132,37 @@ def render(template, **kwargs):
 
 environment = Environment()
 js_environment = jinja2.Environment(loader=Loader('javascript'))
+
+
+@debugcachestats.register()
+@functools.lru_cache()
+def template_config_variables(template: str) -> typing.FrozenSet[str]:
+    """Return the config variables used in the template."""
+    unvisted_nodes = [environment.parse(template)]
+    result = set()  # type: typing.Set[str]
+    while unvisted_nodes:
+        node = unvisted_nodes.pop()
+        if not isinstance(node, jinja2.nodes.Getattr):
+            unvisted_nodes.extend(node.iter_child_nodes())
+            continue
+
+        # List of attribute names in reverse order.
+        # For example it's ['ab', 'c', 'd'] for 'conf.d.c.ab'.
+        attrlist = []  # type: typing.List[str]
+        while isinstance(node, jinja2.nodes.Getattr):
+            attrlist.append(node.attr)  # type: ignore
+            node = node.node  # type: ignore
+
+        if isinstance(node, jinja2.nodes.Name):
+            if node.name == 'conf':  # type: ignore
+                result.add('.'.join(reversed(attrlist)))
+            # otherwise, the node is a Name node so it doesn't have any
+            # child nodes
+        else:
+            unvisted_nodes.append(node)
+
+    from qutebrowser.config import config
+    for option in result:
+        config.instance.ensure_has_opt(option)
+
+    return frozenset(result)
